﻿

function RaidTracker:OnLoadCustom( frame )
	UIPanelWindows[frame:GetName()] = { area = "left", pushable = 1, whileDead = 1 }

	-- Register events
	frame:RegisterEvent("PLAYER_ENTERING_WORLD")
	frame:RegisterEvent("PARTY_MEMBERS_CHANGED")
	frame:RegisterEvent("RAID_ROSTER_UPDATE")
	frame:RegisterEvent("CHAT_MSG_LOOT")
	frame:RegisterEvent("CHAT_MSG_SYSTEM")
	frame:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
	frame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	frame:RegisterEvent("CHAT_MSG_MONSTER_YELL")
	frame:RegisterEvent("CHAT_MSG_MONSTER_EMOTE")
	frame:RegisterEvent("UNIT_HEALTH")
	frame:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	frame:RegisterEvent("ZONE_CHANGED")
	frame:RegisterEvent("ZONE_CHANGED_INDOORS")
	frame:RegisterEvent("UPDATE_INSTANCE_INFO")

	SlashCmdList["RAIDTRACKER"] = RaidTracker_CommandOptions
	SLASH_RAIDTRACKER1 = "/raidtracker"
	SLASH_RAIDTRACKER2 = "/rt"
end

function RaidTracker:OnVariablesLoaded( frame )
	self.L = self._acelib.L

    getglobal(frame:GetName() .. "HeaderIcon"):SetTexture( self._meta.icon )
	getglobal(frame:GetName() .. "HeaderTitle"):SetText( self._meta.title )
	getglobal(frame:GetName() .. "HeaderVersion"):SetText( self._options.Color .. "v" .. self._meta.version )  -- .. " "..self._meta.locale.."|r" )

--	frame:CreateTitleRegion()
--	frame:GetTitleRegion():SetAllPoints()
end


function RaidTracker:OnPlayerLogin( frame )
	local _inst = self._acelib
	if _inst and _inst.menu then
		_inst.menu.args.itemoptions = {
			type = "execute", order = 11,
			name = _inst.L["Item Options"], desc = _inst.L["Show Item Options"],
			passValue = _inst.instance, func = function(self) self:ExecChatCommandAce("io") end,
		};
	end
end


function RaidTracker_CommandOptions( msg )
	RaidTracker:CommandOptions(msg)
end

function RaidTracker:CommandOptions( msg )
	local frame = RaidTrackerFrame
	local db, options, L = self._db, self._options, self.L;

	local _, _, command, args = string.find(strtrim(msg), "(%w+)%s?(.*)")
	if command then	command = strlower(command); end
	if args then args = strtrim(args) if (strlen(args)==0) then args = nil end end

	if command == "debug" then
		if args=="1" then
			options.DebugMode = 1
		elseif args=="0" then
			options.DebugMode = 0
		end
		self:Print("Debug Mode: ".. ((options.DebugMode and "Enabled") or "Disabled"))
	elseif command == "addwipe" then
		self:AddWipeDB()
	elseif command == "deleteall" then
		self:Print(L:F("Deleting %d raids", #db.Log))
		self:TruncateSessionDB()
		KA_RaidTracker_Update()
		KA_RaidTracker_UpdateView()
	elseif command == "additem" then
		local linkMatch = "|c%x+|Hitem:[-%d:]+|h%[.-%]|h|r"
		local _, _, item, player = string.find(args or "", "("..linkMatch..")%s+(%w+)");
		if not item or not player then
			self:Print("AddItem: Must supply an item link and a player name.")
		elseif not frame.selected then
			self:Print("AddItem: There is no raid selected")
		else
			if self:AddLootItemDB(frame.selected, item, player) then
				self:Print(L:F("Added %s to the selected raid.", item))
			else
				self:Print("AddItem: Could not add "..item)
			end
		end
		KA_RaidTracker_Update();
		KA_RaidTracker_UpdateView();
	elseif command == "join" or command == "leave" then
		if not frame.selected then
			self:Print(command..": There is no raid selected");	return;
		end
		if args and strlen(args) > 0 then
			RT_JoinLeaveFrameNameEB:SetText(args);
		end
		RT_Dialog:Show("RT_JoinLeaveFrame", command, nil, frame.selected)
	elseif command == "bossnext" then
		if not options.CurrentRaid then
			self:Print(command..": Must be a current open raid.");	return;
		end
		RT_Dialog:Show("RT_EditBossFrame", "boss", "next", options.CurrentRaid)
	elseif command == "io" then
--		local checks = { "item:(%d+):","(%d+)%s?" };
		local checks = { "item:(%d+):" }
		for i,match in pairs(checks) do
			for itemid in string.gmatch(args or "",match) do
				itemid = tonumber(itemid);
				if db.ItemOptions[itemid] then
					self:Print(L:F("Item %d is already in the Item Options list.", itemid))
				else
--					db.ItemOptions[itemid] = {id = itemid}
					RT_ItemOptionsFrame.itemid = itemid
					RT_ItemOptions:Update()
					self:Print(L:F("Added %d to the Item Options list.", itemid))
				end
			end
		end
		if not args then
			RT_Dialog:ShowToggle(RT_ItemOptionsFrame);
		end	
	elseif command == "options" or command == "o" then
		RT_Dialog:ShowToggle(RT_OptionsFrame);
	elseif command then
		if not self._acelib.instance:OnCommandOptionsAce(msg) then
			self:Print("/rt - Shows the main window.")
			self:Print("/rt options|o - Shows Options window")
			self:Print("/rt io - Shows the ItemOptions window")
			self:Print("/rt io [ITEMLINK|ITEMID]... - Adds items to ItemOptions window")
			self:Print("/rt additem [ITEMLINK] [PLAYER] - Adds a loot item to the selected raid")
			self:Print("/rt join [PLAYER] - Add a player to the selected raid")
			self:Print("/rt leave [PLAYER] - Remmoves a player from the selected raid")
			self:Print("/rt deleteall - Deletes all raids")
			self:Print("/rt debug 1|0 - Enables/Disables debug mode")
			self:Print("/rt addwipe - Adds a Wipe with the current timestamp")
			self._acelib.instance:OnCommandOptionsAce("helpace")
		end
	else
		if frame:IsVisible() then
			HideUIPanel(frame)
		else
			ShowUIPanel(frame)
		end
	end
end


function RaidTracker:OnEventCustom( frame, event, ... )
	local db, options, L = self._db, self._options, self.L;

	if event == "RAID_ROSTER_UPDATE" or (event == "PARTY_MEMBERS_CHANGED" and GetNumRaidMembers() == 0) then
		local count, type = self:GetAttendeeCount(), self:GetAttendeesType();
		if count == 1 and options.CurrentRaid then
			self:EndSessionDB();										-- if a raid and no peoples, den mark them all lefted
		elseif not options.CurrentRaid and self:CanAutoCreateLog() then
			self:CreateSessionDB();									-- if not a raid and peoples der and is okies to make raid by self, den makes a raid
		elseif options.CurrentRaid then
			self:UpdateSessionDB();
		else
			return
		end
		KA_RaidTracker_Update();
		KA_RaidTracker_UpdateView();

	elseif event == "CHAT_MSG_LOOT" then
		if not options.CurrentRaid and self:CanAutoCreateLog() then
			self:CreateSessionDB();									-- if not a raid and peoples der and is okies to make raid by self, den makes a raid
		end
		if options.CurrentRaid then
			local sPlayer, sLink, iCount, bFilter;					-- job here is to get player, link, item count, and if to engage item filters
			if		string.find(arg1, L.ReceivesLoot1) then
				_, _, sPlayer, sLink = string.find(arg1, L.ReceivesLoot1);
			elseif	string.find(arg1, L.ReceivesLoot2) then
				_, _, sLink = string.find(arg1, L.ReceivesLoot2);
				sPlayer = YOU;
			elseif	string.find(arg1, L.ReceivesLoot3) then
				_, _, sPlayer, sLink, iCount = string.find(arg1, L.ReceivesLoot3);
			elseif	string.find(arg1, L.ReceivesLoot4) then
				_, _, sLink, iCount = string.find(arg1, L.ReceivesLoot4);
				sPlayer = YOU;
			end
			if sPlayer and sPlayer == YOU then
				sPlayer = UnitName("player");
			end
			self:Debug("CHAT_MSG_LOOT", "arg1",arg1, "results",sPlayer,sLink,iCount);
			if not sLink or not sPlayer then return; end

			self:AddLootItemDB( options.CurrentRaid, sLink, sPlayer, iCount );
		end
		KA_RaidTracker_Update();
		KA_RaidTracker_UpdateView();

	elseif event == "UPDATE_MOUSEOVER_UNIT" then
		if options.AutoEvent ~= 1 then return end

		local unit, boss = UnitName("mouseover");
		if not UnitIsFriend("mouseover", "player") and not UnitInRaid("mouseover") and not UnitInParty("mouseover") then
			--self:Debug("possible mouseover unit update", unit);
			if db.Bosses[unit] then
				if db.Bosses[unit] ~= "IGNORE" then
					boss = db.Bosses[unit];
					options.EventLast = GetTime();
				end
			elseif db.Bosses["DEFAULTBOSS"] and (GetTime() > (options.EventLast + options.EventCooldown)) then
				boss = db.Bosses["DEFAULTBOSS"];
			end
			if boss and options.BossName ~= boss then
				options.BossName = boss;
				self:Print(L:F("AutoEvent update \"%s\".", boss));
			end
		end
	elseif event == "COMBAT_LOG_EVENT_UNFILTERED"  or event == "CHAT_MSG_MONSTER_YELL" or event == "CHAT_MSG_MONSTER_EMOTE" then
		local filter = COMBATLOG_OBJECT_TYPE_NPC + COMBATLOG_OBJECT_REACTION_HOSTILE;
		local timestamp, type, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags = select(1, ...);
--		self:Debug("COMBAT_LOG_EVENT_UNFILTERED", timestamp, type, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags);
	
		if event == "CHAT_MSG_MONSTER_YELL" or event == "CHAT_MSG_MONSTER_EMOTE" then
			if timeStamp == L.BossKill_Yell_Majordomo then
				destName = L["Majordomo Executus"]
			elseif timeStamp == L.BossKill_Yell_Chess_Event then
				destName = L["Chess Event"]
			elseif timeStamp == L.BossKill_Yell_Julianne then
				destName = L["Julianne"]
				julianne_died = true;
			elseif timeStamp == L.BossKill_Yell_Sathrovarr then
				destName = L["Sathrovarr the Corruptor"];
			else
				return
			end
		elseif type~="UNIT_DIED" or bit.band(destFlags, filter) ~= filter then
			return
		end
		
		-- at this point we have a dead hostile mob
		local unit = L.R[destName]
		local boss,update = db.Bosses[unit];
		self:Debug("CHAT_MSG_COMBAT_HOSTILE_DEATH","unit",destName,unit,boss);
		
		if options.AutoEvent == 2 then
			local boss = boss
			if boss then
				if boss == "IGNORE" then
					boss = nil
				end
			elseif db.Bosses["DEFAULTBOSS"] then
				boss = db.Bosses["DEFAULTBOSS"];
			end
			if boss and options.BossName ~= boss then
				options.BossName = boss;
				update = true
				self:Print(L:F("AutoEvent update \"%s\".", boss));
			end
		end

		-- Romulo and Julianne Hack
		if boss == "Romulo and Julianne" then
			if julianne_died == false then boss = nil end;
		end

		if boss and boss ~= "IGNORE" then 
			if not options.CurrentRaid  and self:CanAutoCreateLog() then
				self:CreateSessionDB()
				update = true
			end
			local raid = db.Log[options.CurrentRaid]
			if raid then
				local found;
				for k, v in pairs(raid.Events) do
					if v.boss == boss then found = true; break; end
				end
				if not found then
					update = true
					local tsDate = self:GetTimestamp();
					local la = options.LogAttendees;
					tinsert( raid.Events, {
							boss = boss,
							time = tsDate,
							Inst = self:GetInstanceDB(self.L.R[GetRealZoneText()]),
							attendees = (la==1 or la==3) and self:GetAttendees("names", true, la==3) or nil,
						} );
					self:Print(L:F("Boss Kill for \"%s\" at %s.", L[boss], self:GetDisplayDate(tsDate)));
					if options.AutoEvent == 3 then
						RT_Dialog:Show("RT_EditBossFrame", "boss", "next", options.CurrentRaid);
					end;
					if options.LogGuild == 1 then
						RaidTracker:AddGuildDB();
					end
				end
			end
		end
		if update then 
			KA_RaidTracker_Update();
			KA_RaidTracker_UpdateView();
		end
	elseif event == "ZONE_CHANGED_NEW_AREA" or event == "UPDATE_INSTANCE_INFO" 
		 or event == "ZONE_CHANGED" or event == "ZONE_CHANGED_INDOORS" then
		self:UpdateZoneDB();
		KA_RaidTracker_Update();
		KA_RaidTracker_UpdateView();
	end

	-- end of events that dont require current raid
	if not options.CurrentRaid then
		return

	elseif event == "PLAYER_ENTERING_WORLD" then
		self:UpdateSessionDB();
		KA_RaidTracker_UpdateView();
		KA_RaidTracker_Update();

	elseif event == "CHAT_MSG_SYSTEM" then
		local name = UnitName("player");
		if name and name ~= UKNOWNBEING and name ~= UNKNOWN then
			local tsDate, sPlayer = self:GetTimestamp();
			if string.find(arg1, L.LeftRaid) then
				_, _, sPlayer = string.find(arg1, L.LeftRaid);
			else 
				_, _, sPlayer = string.find(arg1, L.LeftParty);
			end
			if sPlayer and sPlayer ~= name and sPlayer ~= UKNOWNBEING and sPlayer ~= UNKNOWN and db.Online[sPlayer] then
				tinsert(db.Log[options.CurrentRaid].Leave,	{ player = sPlayer, time = tsDate, }	);
				db.Online[sPlayer] = nil;
				self:Debug(sPlayer, "LEFT", self:GetDisplayDate(tsDate));
			end

			KA_RaidTracker_UpdateView();
			KA_RaidTracker_Update();
		end

	elseif event == "UNIT_HEALTH" then
		if InCombatLockdown() or self:GetAttendeesType()~="Raid" or GetTime() < (options.WipeLast + options.WipeCooldown) then return end;

		if options.LogWipe > 1 then  
			local memberdead, count = 0, self:GetAttendeeCount();
			for i = 1, count, 1 do
				if UnitIsDeadOrGhost(self:GetAttendeeUnitId(i)) then
					memberdead = memberdead + 1;
				end
			end
			if memberdead == count then
				self:AddWipeDB();
				RT_AcceptWipeFrame:Hide();
			elseif (memberdead / count) > (options.LogWipe-1)*0.01 then
				RT_AcceptWipeFrame.type = "wipe"
				RT_AcceptWipeFrame:Show();
			end;
		end
	end

end


-- unifies attendees info lookup and raid/party/bgs/arenas
function RaidTracker:CanAutoCreateLog( isZoneChange )										--self:Debug("CanAutoCreateLog")
	local o, t, c, m = self._options, self:GetAttendeesType(), self:GetAttendeeCount(), (isZoneChange) and 3 or 2;
	return ( (t=="Raid" and o.AutoRaid>=m and c>0) or (t=="Battlegroup" and o.AutoBattlegroup>=m and c>0) or
		   (t=="Party" and o.AutoParty>=m and c>1) or (t=="Arena" and o.AutoAreana>=m and c>1) or (o.AutoSolo>=m and c==1) ) 
		   and c or nil;
end

function RaidTracker:GetAttendeesType( )										--self:Debug("GetAttendeesType")
	if IsActiveBattlefieldArena() == 1 then return "Arena" end			
	if GetNumBattlefieldScores() > 0 then return "Battlegroup" end
	if GetNumRaidMembers() > 0 then return "Raid" end
	if GetNumPartyMembers() > 0 then return "Party" end
	return "Solo"
end

function RaidTracker:GetAttendeeCount( )										--self:Debug("GetAttendeeCount")
	local o = self._options;
	if GetNumRaidMembers() > 0 then
		return (o.AutoRaid >= 1 or (GetNumBattlefieldScores() > 0 and o.AutoBattlegroup >= 1)) and GetNumRaidMembers() or 0;
	end
	if GetNumPartyMembers() > 0 then	-- if party logging is off then will return 0 if on a party
		return (o.AutoParty >= 1 or	(IsActiveBattlefieldArena() == 1 and o.AutoAreana >= 1)) and (GetNumPartyMembers() + 1) or 0;
	end
	return (o.AutoSolo >= 1) and 1 or 0;
end

-- these two are very useful but mess for speed
function RaidTracker:GetAttendeeUnitId( i, nRaid, nParty )						--self:Debug("GetAttendeeUnitId", i, nRaid, nParty)
	if type(i)~="number" then return i end
	if (nRaid or GetNumRaidMembers()) > 0 then return "raid"..i end
	return (i > (nParty or GetNumPartyMembers())) and "player" or ("party"..i);
end

function RaidTracker:GetAttendeeInfo( i, isExtended, nRaid, nParty )			--self:Debug("GetAttendeeInfo", i, isExtended, nRaid, nParty)
	local unit = self:GetAttendeeUnitId( i, nRaid, nParty );

	local name, online, zone, level, class;
	if (nRaid or GetNumRaidMembers()) > 0 and type(i) == "number" then
		name,_,_,level,_,class,zone,online = GetRaidRosterInfo(i);	-- name, rank, subgroup, level, class, fileName, zone, online = GetRaidRosterInfo(i);
	else
		name,online,zone = UnitName(unit),UnitIsConnected(unit),self:UnitZone(unit);
	end
	if not isExtended then
		return name, online, zone;
	end

	level = level or UnitLevel(unit);
	if not class then _, class = UnitClass(unit); end
	local sex, guild, _, race = UnitSex(unit), GetGuildInfo(unit), UnitRace(unit);
	return name, online, zone, level, class, sex, guild, race;
end

function RaidTracker:GetAttendees( c, chkOnline, chkZone )	-- jsut needs to be fast, its a bit convoluted, but fast and small
	local t = (type(c)~="number") and { } or nil; 			-- okies, can pass count to make faster, else get count, if not number then is "type"
	local n = self:_GetAttendees( chkOnline, chkZone, ((not t) and c or self:GetAttendeeCount()), t, t and c~="names", GetNumRaidMembers(), GetNumPartyMembers());
	return t or n
end

function RaidTracker:_GetAttendees( chkOnline, chkZone, count, t, ids, nRaid, nParty )	--self:Debug("_GetAttendees", i, isExtended, nRaid, nParty)
	if count == 0 then return 0 end
	local n = 0
	for i = 1, count do
		name, online, zone = self:GetAttendeeInfo(i, false, nRaid, nParty);
		if (name and name ~= UKNOWNBEING and name ~= UNKNOWN) and 		-- check name and itentity
			(not chkZone or not zone or zone==GetRealZoneText()) and	-- should check, haz a zone, zone matches
			(not chkOnline or online) then  							-- should check, is online
				n = n + 1
				if t then t[n] = (ids and i or name) end
		end
	end
	return n
end


function RaidTracker:UpdateSessionDB( isNew, isFast )
	local db, options = self._db, self._options;
	if not options.CurrentRaid then return end
	local raid = db.Log[options.CurrentRaid]
	if not raid then self:EndSessionDB() return end

	-- update player info, join and leave times
	local tAttend, tsDate = self:GetAttendees(), self:GetTimestamp();
	for i, v in pairs(tAttend) do
		local name, online, zone, level, class, sex, guild, race = self:GetAttendeeInfo(i, true);

		if not raid.Players[name] then
			raid.Players[name] = { };
		end
		if options.SaveExtPlayer == 1 then
			local ti = raid.Players[name];
			if race then		ti.race = race; end
			if class then		ti.class = class; end
			if sex then			ti.sex = sex; end
			if level > 0 then	ti.level = level; end
			if guild then		ti.guild = guild; end
		end

		if isNew or online ~= db.Online[name] then						-- this will change status for existing player
			if isNew or online then
				tinsert( raid.Join,  { player = name, time = tsDate } );
			end
			if not online then
				tinsert( raid.Leave, {	player = name, time = tsDate } );
			end
		end

		db.Online[name] = online;
	end
	
	self:UpdateZoneDB()
end


function RaidTracker:GetInstanceDB( zone )
	for i=1, GetNumSavedInstances() do
		local name, id, reset, difficulty = GetSavedInstanceInfo(i);
		if zone == name then
			return { id = id, name = name, reset = reset, difficulty = difficulty  }
		end;
	end
end

function RaidTracker:UpdateZoneDB( )
	local db, options = self._db, self._options;
	if options.AutoZone == 0 or not options.CurrentRaid then return end
	local raid = db.Log[options.CurrentRaid]
	local zone = self.L.R[GetRealZoneText()];	
	if zone and db.Zones[zone] then																-- is a trigger
		if not raid.zone then
			self:Print(self.L:F("Setting current raid zone to %s.", GetRealZoneText()));
			raid.zone = zone;
		elseif zone ~= raid.zone and self:CanAutoCreateLog(true) then							-- has one already
			self:Print(self.L["Creating new raid due to zone change."]);
			self:CreateSessionDB()																-- this wil get called again on create new session, so bail
			return;
		end
	end
	if raid.zone then
		raid.Inst = self:GetInstanceDB(raid.zone)
	end
end


function RaidTracker:AddWipeDB()
	local options = self._options;
	if not options.CurrentRaid then return end
	local raid = self._db.Log[options.CurrentRaid]
	self:Debug("RaidTracker:AddWipeDB");
		
	options.WipeLast = GetTime();
	
	if not raid.Wipes then
		raid.Wipes = { }
	end
	tinsert( raid.Wipes, self:GetTimeStamp() );
	self:Print(self.L["Wipe has been recorded."]);
end

function RaidTracker:AddGuildDB()
	local db, options = self._db, self._options;
	if not options.CurrentRaid then return end
	local raid = db.Log[options.CurrentRaid]
	self:Debug("RaidTracker:AddGuildDB");

	SetGuildRosterShowOffline(false);
	local nDate = self:GetTimestamp();
	for i = 1, GetNumGuildMembers() do
		local name, rank, rankIndex, level, class, zone, group, note, officernote, online = GetGuildRosterInfo(i);
		if online ~= db.Online[name] then
			if online then
				tinsert( raid.Join, { player = name, time = nDate } );
				db.Online[name] = online;
			elseif not online and db.Online[name] then
				tinsert( raid.Leave, { player = name, time = nDate } );
				db.Online[name] = online;
			end
		end
	end
end


function RaidTracker:CreateSessionDB( )
	local db = self._db;
	local options = self._options;
	local tsDate,sRealm = self:GetTimestamp(),GetRealmName();

	-- start new raid
	self:EndSessionDB()
	tinsert( db.Log, 1, {
		Loot = { },
		Join = { },
		Leave = { },
		Players = { },
		Events = { },
		Wipes = nil,
		Inst = nil,
		key = tsDate,
		End = nil,
		zone = nil,
		realm = sRealm,
		note = nil,
		bossnext = nil,
	} );
	options.CurrentRaid = 1;
	self:Print(self.L:F("Joining new raid at %s.", self:GetDisplayDate(tsDate)));
	self:UpdateSessionDB( true );					-- setup player info, join and leave times
end

function RaidTracker:EndSessionDB( )
	local db = self._db;
	local options = self._options;
	
	if options.CurrentRaid then
		local tsTime = self:GetTimestamp();
		local raid = db.Log[options.CurrentRaid];
		self:Print(self.L:F("Ending current raid at %s.", self:GetDisplayDate(tsTime)));
		if raid then
			for k, v in pairs(db.Online) do
				tinsert( raid.Leave, { player = k, time = tsTime, } );
			end
			if not raid.End then
				raid.End = tsTime;
			end
		end
		options.CurrentRaid = nil;
	end

	db.Online = { };
end

function RaidTracker:DeleteSessionDB( raidid )
	local frame = RaidTrackerFrame;
	local db = self._db;
	local options = self._options;

	if frame.selected == raidid then			-- unhook frame if selected raid
		if frame.selected > 1 then
			 frame.selected = frame.selected - 1;
		end
		frame.type = "raids";
	end
	if options.CurrentRaid == raidid then		-- unhook db if current raid
		options.CurrentRaid = nil;
		db.Online = { };
	end

	table.remove(db.Log, raidid);
end

function RaidTracker:TruncateSessionDB( )
	db.Log = { };
	db.Online = { };
	options.CurrentRaid = nil;
end

function RaidTracker:SnapshotSessionDB( )
	local db = self._db;
	local options = self._options;
	if not options.CurrentRaid then return end

	self:Print(self.L["Snapshotting current raid."]);
	local raid = db.Log[options.CurrentRaid];
	local tsDate = self:GetTimestamp();	
	local t = {
		key = tsDate,
		End = tsDate,
		zone = raid.zone,
	}
	for k, v in pairs(raid) do
		if type(v) == "table" then t[k] = { } end
	end
	for k, v in pairs(db.Online) do
		t.Players[k] = { };
		tinsert(t.Join, { player = k, time = tsDate } );
		tinsert(t.Leave, { player = k, time = tsDate } );
	end
	tinsert(db.Log, 2, t);
end

function RaidTracker:CreateLootItemDB( frame, sLink, sPlayer, iCount, ioGetCost )
	iCount = tonumber(iCount);
	local options = self._options;
	local sColor, sItem, sName = self:GetLinkMeta(sLink);
	if not sItem then return end
	local sNote, sCosts;

	if ioGetCost then												-- get cost from another system
		local _, _, s0, s1, s2, s3 = string.find(sItem, "^(%d+):(%d+):(%d+):(%d+)");
		if		DKPValues and DKPValues[tostring(s0)] then			-- AdvancedItemTooltip
			sCosts = tonumber(DKPValues[tostring(s0)]);
		elseif	HDKP_GetDKP then									-- HoB_DKP
			sCosts = tonumber(HDKP_GetDKP(s0, s1, s2, s3));
		end
		if sCosts == 0 then sCosts = nil end;
	end

	local name, link, quality, iLevel, minLevel, class, subclass, maxStack, invtype, icon = GetItemInfo("item:"..sItem);
	if icon then
		_, _, icon = string.find(icon, "^.*\\(.*)$");
	end
	local lootItem = {
		player = sPlayer,
		item = {
			c = sColor,
			id = sItem,
			tooltip = (options.SaveTooltips == 1) and self:GetItemTooltip(sItem) or nil,
			name = sName,
			icon = icon,
			count = (iCount and iCount > 1) and iCount or nil;
			class = class,
			subclass = subclass,
		},
		costs = sCosts,
		boss = (options.AutoEvent >= 1) and options.BossName or nil,
		note = sNote,
		zone = GetRealZoneText(),
		time = self:GetTimestamp(),
		attendees = (options.LogAttendees == 2) and self:GetAttendees("names", true) or nil,
	}
	return lootItem;
end

function RaidTracker:AddLootItemDB( raidid, sLink, sPlayer, iCount )
	if not raidid then return end
	iCount = iCount and tonumber(iCount);
	local frame, db, options = RaidTrackerFrame, self._db, self._options;
	local items = db.Log[raidid].Loot;
	self:Debug("AddLootItemDB","rid",raidid,"link",sLink,"player",sPlayer)

	-- check via Options
	local sColor, sItem, sName = self:GetLinkMeta(sLink);
	local rarity = self._lookup.ColorToRarity[sColor];
	local _, _, itemid = string.find(sItem, "^(%d+):%d+:%d+:%d+");
	local io = db.ItemOptions[tonumber(itemid)];
	local ioGroup, ioCostAsk, ioGetCost;
	if (io and io.Track==0)	or ((not io or not io.Track) and not
		(rarity >= options.LogItems) ) then return; end
	ioGroup = (io and io.Group==1) or ((not io or not io.Group) and
		(options.GroupItems~=0 and rarity <= options.GroupItems) );
	ioGetCost = (io and io.CostGet==1) or ((not io or not io.CostGet) and
		(options.CostGet~=0 and rarity >= options.CostGet) );
	ioCostAsk = (io and io.CostAsk==1) or ((not io or not io.CostAsk) and
		(options.CostAsk~=0 and rarity >= options.CostAsk) );

	-- add loot item
	local t;
	if ioGroup then
		for k, v in pairs(items) do
			if v.item.name == sName and v.player == sPlayer then
				v.item.count = (v.item.count or 1) + ((iCount and iCount>0) and iCount or 1);
				t = v;
				break;
			end
		end
	end
	if not t then
		t = self:CreateLootItemDB(frame, sLink, sPlayer, iCount, ioGetCost);
		t.id = #items + 1
		tinsert(items, t.id, t);
		if ioCostAsk then									-- code and idea from tlund
			RT_Dialog:Show("RT_EditCostFrame", "itemcost", nil, raidid, t.id);
		end
	end
	return t;
end

function RaidTracker:GetLinkMeta( sLink )
	if not sLink then return end
	local _, _, sColor, sItemStr, sName = string.find(sLink, "|c(%x+)|Hitem:([-%d:]+)|h%[(.-)%]|h|r");
	return sColor, sItemStr, sName;
end

function RaidTracker:GetDisplayDate( t )
	return t and self:GetDate(t, self._options.Timezone) or nil;
end

function RaidTracker:GetTimestamp( s, ts )
	return self:GetTime(s, ts or self._update.timediff);
end

function RaidTracker:GetRaidTitle(raidid, hideid, showzone, shortdate, color)
	local db, L = self._db, self.L;
	if not raidid then return L["Title"]; end
	local raid = db.Log[raidid];
	
	local s = self:GetDisplayDate(raid.key);
	local _, _, mon, day, year, hr, min, sec = string.find(s, "(%d+)/(%d+)/(%d+) (%d+):(%d+):(%d+)");
	if mon then
		s = "";
		local months = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec", };
		if not hideid then
			s = s .. "[" .. (#db.Log - raidid + 1) .. "] ";
		end

		if not shortdate then
			s = s .. L[months[tonumber(mon)]] .. " " .. day .. " '" .. year .. ", " .. hr .. ":" .. min;
		else
			s = s .. mon .. "/" .. day .. " " .. hr .. ":" .. min;
		end
		if color then s = s .. "|c00ffffff" end
		if showzone and raid.zone then
			s = s .. " " .. L[raid.zone];
		end
		if color then s = s .. "|r" end
	end

	return s;
end

function RaidTracker:GetStatusText( frame )
	local db, o, L = self._db, self._options, self.L;
	local raid = o.CurrentRaid and db.Log[o.CurrentRaid]
	
	-- build status text
	local c = self:GetAttendeeCount()
	local cOL, cZ = self:GetAttendees(c, true), self:GetAttendees(c, true, true);
	local sc =  c>0 and (" (" .. cOL .. ((cOL~=c) and ("/" .. c) or "") .. ")") or "";
	local sz = GetRealZoneText() .. (GetSubZoneText()~="" and (": "..GetSubZoneText()) or "");
	local szc =  (cZ>0 and cZ~=cOL) and (" ("..cZ..")") or "";
	local boss = (o.BossName and o.BossName~="Trash mob") and (" ["..L[o.BossName].."]") or ""
	return L[self:GetAttendeesType()] .. sc .. " - " .. sz .. szc .. boss;
end


--[[ *****************************************************************************************************
	******************************************************************************************************
	******************************************************************************************************
	****************************************************************************************************** ]]


function KA_RaidTracker_Update(frame)
	local frame = RaidTrackerFrame;
	local db = KARaidTrackerDB;
	local options = db.Options;
	
	-- main frame top buttons
	if options.CurrentRaid then
		RaidTrackerFrameEndRaidButton:Enable();
		RaidTrackerFrameSnapshotButton:Enable();
	else
		RaidTrackerFrameEndRaidButton:Disable();
		RaidTrackerFrameSnapshotButton:Disable();
	end
	if RaidTracker:GetAttendeeCount() > 0 then
		RaidTrackerFrameNewRaidButton:Enable();
	else
		RaidTrackerFrameNewRaidButton:Disable();
	end
	if frame.LastPage and #frame.LastPage > 0 then
		RaidTrackerFrameBackButton:Enable();
	else
		RaidTrackerFrameBackButton:Disable();
	end

	-- process
	local maxitems = #db.Log;
	if maxitems <= 0 then
		frame.type = "raids";
		frame.selected = nil;
		RT_DetailFramePlayers:Hide();
		RT_DetailFrameItems:Hide();
		RT_DetailFrameRaids:Hide();
		RT_DetailFrameEvents:Hide();
		RaidTrackerFrameDeleteButton:Disable();
		RaidTrackerFrameViewButton:Disable();
	else
		RaidTrackerFrameDeleteButton:Enable();
		frame.type = frame.type or "raids";

		local type = frame.type;
		if type == "raids" or type == "items" then
			frame.selected = frame.selected or 1;
			if not db.Log[frame.selected] then
				frame.selected = maxitems;
			end
			if #db.Log[frame.selected].Loot == 0 then
				RaidTrackerFrameViewButton:Disable();
			else
				RaidTrackerFrameViewButton:Enable();
			end
			RaidTrackerFrameViewButton:SetText( (type == "raids" and "View Items") or "View Players");
		elseif type == "player" or type == "playeritems" then
			local hasItem;
			for k, v in pairs(db.Log) do
				for k1, v1 in pairs(v.Loot) do
					if v1.player == frame.player then hasItem = 1;	break; end
				end
				if hasItem then	break; end
			end
			if not hasItem then
				if type == "playeritems" then
					frame.type = "player";
					KA_RaidTracker_Update();
					KA_RaidTracker_UpdateView();
					return;
				end
				RaidTrackerFrameViewButton:Disable();
			else
				RaidTrackerFrameViewButton:Enable();
			end
			RaidTrackerFrameDeleteButton:Disable();
			RaidTrackerFrameViewButton:SetText((type == "player" and "View Loot") or "View Raids" );
		elseif type == "itemhistory" or type == "events" then
			RaidTrackerFrameViewButton:Disable();
			RaidTrackerFrameDeleteButton:Disable();
		end
	end

	-- ScrollFrame update	
	KA_RaidTracker_UpdateScroll()
	RaidTrackerFrameHeaderText:SetText(RaidTracker:GetStatusText(frame));
end

function KA_RaidTracker_UpdateScroll( )
	local frame = RT_LogFrame
	local db = KARaidTrackerDB;
	
	local maxitems = #db.Log;
	FauxScrollFrame_Update(frame, maxitems, 6, 15)
	for i=1, 6 do
		local id = i + FauxScrollFrame_GetOffset(frame);
		local sline = "RT_LogFrameLine" .. i
		local line = getglobal(sline);

		if id > maxitems then
			line.raidid = nil;
			line:Hide();
		else
			line.raidid = id;
			local raidTitle = RaidTracker:GetRaidTitle(id, nil, 1, 1, true) or "";
			local raidTag = db.Log[id].note;
			line:SetText( raidTitle .. (raidTag and (" ("..raidTag..")") or "") );
			if RaidTrackerFrame.selected == id then
				getglobal(sline.."MouseOver"):Show()
			else
				getglobal(sline.."MouseOver"):Hide()
			end
			line:Show();
		end
	end
	frame:Show();
end


function KA_RaidTracker_SortPlayerRaids( id )
	local sortopt = KARaidTrackerDB.SortOptions
	if RaidTrackerFrame.type == "itemhistory" then
		local table = { "name", "looter" };

		if sortopt["itemhistorymethod"] == table[id] then
			if sortopt["itemhistoryway"] == "asc" then
				sortopt["itemhistoryway"] = "desc";
			else
				sortopt["itemhistoryway"] = "asc";
			end
		else
			sortopt["itemhistoryway"] = "asc";
			sortopt["itemhistorymethod"] = table[id];
		end
	else
		if sortopt["playerraidway"] == "asc" then
			sortopt["playerraidway"] = "desc";
		else
			sortopt["playerraidway"] = "asc";
		end
	end
	KA_RaidTracker_UpdateView();
end

function KA_RaidTracker_SortItemBy(id)
	local sortopt = KARaidTrackerDB.SortOptions
	local table = { "name", "looted", "looter", "rarity" }
	local prefix = (RaidTrackerFrame.type == "playeritems") and "player" or ""
	
	if sortopt[prefix.."itemsmethod"] == table[id] then
		if sortopt[prefix.."itemsway"] == "asc" then
			sortopt[prefix.."itemsway"] = "desc"
		else
			sortopt[prefix.."itemsway"] = "asc"
		end
	else
		sortopt[prefix.."itemsmethod"] = table[id]
		sortopt[prefix.."itemsway"] = "asc"
	end
	KA_RaidTracker_UpdateView()
end

function KA_RaidTracker_SortBy(id)
	local sortopt = KARaidTrackerDB.SortOptions
	local table = { "name", "join", "leave" };
	if sortopt["method"] == table[id] then
		if sortopt["way"] == "asc" then
			sortopt["way"] = "desc"
		else
			sortopt["way"] = "asc"
		end
	else
		sortopt["method"] = table[id];
		if table[id] ~= "leave" then
			sortopt["way"] = "asc"
		else
			sortopt["way"] = "desc"
		end
	end
	KA_RaidTracker_UpdateView();
end

function KA_RaidTracker_CompareItems(a1, a2)
	local sortopt = KARaidTrackerDB.SortOptions
	local lookup = RaidTracker._lookup;

	local filter, method, way = sortopt.itemsfilter, sortopt.itemsmethod, sortopt.itemsway;
	if RaidTrackerFrame.type == "playeritems" then
		filter, method, way = sortopt.playeritemsfilter, sortopt.playeritemsmethod, sortopt.playeritemsway;
	end

	if lookup.ColorToRarity[a1.item.c] < filter then
		return false;
	elseif lookup.ColorToRarity[a2.item.c] < filter then
		return true;
	end

	if method == "name" then
		local c1, c2 = a1.item.name, a2.item.name;
		if c1 == c2 then
			c1, c2 = a1.player, a2.player;
		end
		if way == "asc" then
			return c1 < c2;
		else
			return c1 > c2;
		end
	elseif method == "looter" then
		local c1, c2 = a1.player, a2.player;
		if c1 == c2 then
			c1, c2 = lookup.ColorToRarity[a2.item.c], lookup.ColorToRarity[a1.item.c];
			if c1 == c2 then
				c1, c2 = a1.item.name, a2.item.name;
			end
		end
		if way == "asc" then
			return c1 < c2;
		else
			return c1 > c2;
		end
	elseif method == "looted" then
		if way == "asc" then
			return a1.time < a2.time;
		else
			return a1.time > a2.time;
		end
	else
		local c1, c2 = lookup.ColorToRarity[a1.item.c], lookup.ColorToRarity[a2.item.c];
		if c1 == c2 then
			c1, c2 = a1.item.name, a2.item.name;
			if c1 == c2 then
				c1, c2 = a1.player, a2.player;
			else
				return c1 < c2;
			end
		end
		if way == "asc" then
			return c1 < c2;
		else
			return c1 > c2;
		end
	end
end

function KA_RaidTracker_UpdateView( )
	local frame = RaidTrackerFrame
	local self = RaidTracker;
	local db, options, L = self._db, self._options, self.L
	local sortopt = db.SortOptions;
	local lookup = RaidTracker._lookup	
	
	local raidid = frame.selected;
	local raid = raidid and db.Log[raidid]
	if frame.type == "events" then
		RaidTrackerFrameView2Button:SetText("View Raid");
	else
		RaidTrackerFrameView2Button:SetText("View Events");
		if not raidid or (not raid.Events or #raid.Events==0) then
			RaidTrackerFrameView2Button:Disable();
		else
			RaidTrackerFrameView2Button:Enable();
		end
	end
	if #db.Log <= 0 then return end

	if frame.type == "raids" or not frame.type then
		RT_DetailFramePlayers:Show();
		RT_DetailFrameRaids:Hide();
		RT_DetailFrameItems:Hide();
		RT_DetailFrameEvents:Hide();
		
		local players = { };
		if raid then
			local index = { };		
			for k, v in pairs(raid.Join) do
				if v.player then
					local id = index[v.player];			
					if not id or v.time < players[id].join then
						local t = { join = v.time, name = v.player };
						if index[v.player] then
							players[id] = t;
						else
							tinsert( players, t );
							index[v.player] = #players;
						end
					end
					id = index[v.player];
					local player = players[id]
					if not player.lastjoin or player.lastjoin < v.time then
						player.lastjoin = v.time;
					end
				end
			end
			for k, v in pairs(raid.Leave) do
				local id = index[v.player];
				if id then
					local player = players[id]
					if (not player.leave or v.time > player.leave) and v.time >= player.lastjoin then
						player.leave = v.time;
					end
				end
			end
			for k, v in pairs(players) do
				if not v.leave then
					players[k].leave = 99999999999;			-- Very ugly hack, I know :(
				end
			end
			
			local method, way = sortopt.method, sortopt.way
			table.sort(	players,
				function(a, b)
					if way == "asc" then
						return (a[method] < b[method]);
					else
						return (a[method] > b[method]);
					end
				end
			);
			
			RT_DetailFramePlayers.raidid = raidid;
			RT_DetailFramePlayers.players = players;
			KA_RaidTracker_DetailScrollFramePlayers_Update();
		end
		RT_DetailFramePlayersText:SetText("Participants (" .. #players .. ")");
		RT_DetailFramePlayers:Show();
	elseif frame.type == "items" then
		RT_DetailFramePlayers:Hide();
		RT_DetailFrameRaids:Hide();
		RT_DetailFrameItems:Show();
		RT_DetailFrameEvents:Hide();
		local nitems, nhidden = 0, 0;
		
		if raid then
			local keystoremove = {};
			local loot = {}
			for k, v in pairs(raid.Loot) do
				loot[k] = v;
			end
			table.sort( loot, KA_RaidTracker_CompareItems );
			
			for k, v in pairs(loot) do
				if ( not v.item.tooltip or #v.item.tooltip == 0 ) and options.SaveTooltips == 1
						and GetItemInfo("item:" .. v.item.id) then
					v.item.tooltip = RaidTracker:GetItemTooltip(v.item.id);
				end
				if lookup.ColorToRarity[v.item.c] >= sortopt.itemsfilter then
					nitems = nitems + 1;
				else
					tinsert(keystoremove, k);
					nhidden = nhidden + 1;
				end
			end
			for k, v in pairs(keystoremove) do
				table.remove(loot, v);
			end
			RT_DetailFrameItems.raidid = raidid;
			RT_DetailFrameItems.loot = loot;
			RT_DetailFrameItems.type = frame.type;

			KA_RaidTracker_DetailScrollFrameItems_Update();
			RT_DetailFrameItems:Show();
		end
		RT_DetailFrameItemsText:SetText("Items ("..nitems.. ((nhidden~=0) and ("/"..(nhidden + nitems)) or "") .."):");
		RT_Button:DetailFrameItems_RarityDropdown_Update();

	elseif frame.type == "player" then
		RT_DetailFramePlayers:Hide();
		RT_DetailFrameRaids:Show();
		RT_DetailFrameItems:Hide();
		RT_DetailFrameEvents:Hide();
		RT_DetailFrameRaidsTabLooter:Hide();
		RT_DetailFrameRaidsTab1:SetWidth(300);
		RT_DetailFrameRaidsTab1Middle:SetWidth(290);
		local name = frame.player;

		local raids = { };
		for k, v in pairs(db.Log) do
			for k1, v1 in pairs(v.Join) do
				if v1.player == name then
					tinsert(raids, { k, v });
					break;
				end
			end
		end

		local way = sortopt.playerraidway
		table.sort(raids, function(a, b)
				if way == "asc" then
					return a[2].key < b[2].key;
				else
					return a[2].key > b[2].key;
				end
			end
		);
		RT_DetailFrameRaids.data = raids;
		RT_DetailFrameRaids.name = name;
		KA_RaidTracker_DetailScrollFramePlayer_Update();
		RT_DetailFrameRaidsText:SetText(name .. "'s Raids (" .. #raids .. "):");
		RT_DetailFrameRaids:Show();

	elseif frame.type == "itemhistory" then
		RT_DetailFramePlayers:Hide();
		RT_DetailFrameRaids:Show();
		RT_DetailFrameItems:Hide();
		RT_DetailFrameEvents:Hide();
		RT_DetailFrameRaidsTabLooter:Show();
		RT_DetailFrameRaidsTab1:SetWidth(163);
		RT_DetailFrameRaidsTab1Middle:SetWidth(155);

		local name, totalItems = frame.itemname, 0;
		local loot = { };
		for k, v in pairs(db.Log) do
			for k1, v1 in pairs(v.Loot) do
				if v1.item.name == name then
					tinsert(loot, { k, v, v1 });
					totalItems = totalItems + (v1.item.count or 1);
				end
			end
		end

		local method, way = sortopt[frame.type.."method"], sortopt[frame.type.."way"]
		table.sort( loot, function(a, b)
			if method == "looter" then
				if way == "asc" then
					return a[3].player < b[3].player;
				else
					return a[3].player > b[3].player;
				end
			else
				if way == "asc" then
					return a[2].key < b[2].key;
				else
					return a[2].key > b[2].key;
				end
			end
		end	);

		RT_DetailFrameRaids.data = loot;
		RT_DetailFrameRaids.name = name;
		KA_RaidTracker_DetailScrollFramePlayer_Update();
		RT_DetailFrameRaidsText:SetText(name .. " (" .. #loot .. "/" .. totalItems .. "):");
		RT_DetailFrameRaids:Show();

	elseif frame.type == "events" then
		RT_DetailFramePlayers:Hide();
		RT_DetailFrameRaids:Hide();
		RT_DetailFrameEvents:Show();
		RT_DetailFrameItems:Hide();
		RT_DetailFrameEventsTabBoss:Show();
		RT_DetailFrameEventsTab1:SetWidth(163);
		RT_DetailFrameEventsTab1Middle:SetWidth(155);

		local events = { };
		if raid and raid.Events then
			for k, v in pairs(raid.Events) do
				tinsert(events, v);
			end
		end
		
		RT_DetailFrameEvents.raidid = raidid;
		RT_DetailFrameEvents.events = events;
		KA_RaidTracker_DetailScrollFrameEvents_Update();
		RT_DetailFrameEventsText:SetText(self:GetRaidTitle(raidid, false, true, false));
		RT_DetailFrameEvents:Show();
		

	elseif frame.type == "playeritems" then
		RT_DetailFramePlayers:Hide();
		RT_DetailFrameRaids:Hide();
		RT_DetailFrameItems:Show();
		RT_DetailFrameEvents:Hide();
		local name = frame.player;

		local loot = { };
		for k, v in pairs(db.Log) do
			for k1, v1 in pairs(v.Loot) do
				if v1.player == name then
					tinsert( loot, {
							note = v1.note,
							player = v1.player,
							time = v1.time,
							item = v1.item,
							ids = { k, k1 }
						}
					);
				end
			end
		end

		local nitems, nhidden = 0, 0;
		local keystoremove = {};
		table.sort( loot, KA_RaidTracker_CompareItems );
		for k, v in pairs(loot) do
			if lookup.ColorToRarity[v.item.c] >= sortopt.playeritemsfilter then
				nitems = nitems + 1;
			else
				tinsert(keystoremove, k);
				nhidden = nhidden + 1;
			end
		end
		for k, v in pairs(keystoremove) do
			table.remove(loot, v);
		end
		RT_DetailFrameItems.raidid = raidid;
		RT_DetailFrameItems.loot = loot;
		RT_DetailFrameItems.type = frame.type;

		KA_RaidTracker_DetailScrollFrameItems_Update();
		RT_DetailFrameItemsText:SetText(name .. "'s Loot (" .. nitems .. ((nhidden~=0) and ("/"..(nhidden+nitems)) or "") .. "):");

		RT_Button:DetailFrameItems_RarityDropdown_Update( );
	end
end


function KA_RaidTracker_DetailScrollFramePlayers_Update()
	local frame = RT_DetailFramePlayers
	local self = RaidTracker;
	local db, options, L = self._db, self._options, self.L
	
	local raid = db.Log[frame.raidid]
	local maxlines = #frame.players;
	FauxScrollFrame_Update(frame, maxlines, 11, 18);
	for i = 1, 11 do
		local sline = "RT_DetailFramePlayersLine"..i;
		local line = getglobal(sline);
		local id = i + FauxScrollFrame_GetOffset(frame);		
		if id > maxlines then
			line:Hide();
		else
			player = frame.players[id];
			line.raidid = frame.raidid;
			line.playerid = player.name;
			line.raidtitle = self:GetRaidTitle(frame.raidid, 1);
			if getglobal(sline .. "Name") then
				getglobal(sline .. "Name"):SetText(player.name);
				getglobal(sline .. "Number"):SetText( (id < 10) and ("  "..id) or id );
				getglobal(sline .. "Join"):SetText( date((options.TimeFormat==1) and "%H:%M" or "%I:%M%p", player.join) );
				getglobal(sline .. "Leave"):SetText("");
				if player.leave ~= 99999999999 then
					getglobal(sline .. "Leave"):SetText( date((options.TimeFormat==1) and "%H:%M" or "%I:%M%p", player.leave) );
				end
				local c = raid.Players[player.name];  c = (c and c.note) and 1 or 0.5;
				getglobal(sline .. "NoteButtonNormalTexture"):SetVertexColor(c, c, c);
				getglobal(sline .. "NoteButton"):Show();
				getglobal(sline .. "DeleteButton"):Show();
				line:Show();
			end
		end
	end
	frame:Show();
end

function KA_RaidTracker_DetailScrollFrameItems_Update()
	local frame = RT_DetailFrameItems;
	local self = RaidTracker;
	local db, options, L = self._db, self._options, self.L

	local raid = db.Log[frame.raidid]
	local loot = frame.loot;
	local maxlines = #loot;
	FauxScrollFrame_Update(frame, maxlines, 5, 41);
	for i=1, 5 do
		local sline = "RT_DetailFrameItemsLine" .. i;
		local line = getglobal(sline);
		local id = i + FauxScrollFrame_GetOffset(frame);
		if id > maxlines then
			line:Hide();
		else
			local item = loot[id]
			local count = item.item.count
			line.itemname = item.item.name;
			if count and count > 1 then
				getglobal(sline .. "Count"):Show();
				getglobal(sline .. "Count"):SetText(count);
			else
				getglobal(sline .. "Count"):Hide();
			end
			getglobal(sline .. "IconTexture"):SetTexture("Interface\\Icons\\" .. item.item.icon or options.ItemIcon);
			local c = item.item.c;
			getglobal(sline .. "Description"):SetText("|c" .. ((c == "ff1eff00") and "ff005F00" or c) .. item.item.name);	-- tone down the green
			c = item.note and 1 or 0.5
			getglobal(sline .. "NoteNormalTexture"):SetVertexColor(c, c, c);
			if frame.type == "items" then
				line.raidid = frame.raidid;
				line.itemid = item.id;
				getglobal(sline .. "Looted"):SetText(L:F("Looted by: %s", self:ToStr(nil,40,item.player)));
			elseif frame.type == "playeritems" then
				line.raidid = item.ids[1];
				line.itemid = item.ids[2];
				getglobal(sline .. "Looted"):SetText(L:F("Looted %s", self:GetRaidTitle(item.ids[1], 1)));
			end
			line:Show();
		end
	end
	frame:Show();
end

function KA_RaidTracker_DetailScrollFramePlayer_Update()
	local frame = RT_DetailFrameRaids;
	local self = RaidTracker;
	local db, options, L = self._db, self._options, self.L

	local name = frame.name;
	local data = frame.data;
	local maxlines = #data;
	FauxScrollFrame_Update(frame, maxlines, 11, 18);
	for i=1, 11 do
		local sline = "RT_DetailFrameRaidsLine" .. i;
		local line = getglobal(sline);
		local id = i + FauxScrollFrame_GetOffset(frame);
		if id > maxlines then
			line:Hide();
		else
			local player = data[id];
			local iNumber = #db.Log - player[1] + 1;
			
			line.raidid = player[1];
			getglobal(sline .. "Number"):SetText((iNumber < 10) and ("  " .. iNumber) or iNumber);
			getglobal(sline .. "Name"):SetText(self:GetRaidTitle(player[1], 1));
			if RaidTrackerFrame.type == "player" then
				line.playerid = name;
				line.raidtitle = self:GetRaidTitle(player[1], 1);
				getglobal(sline .. "HitAreaLeft"):Hide();
				getglobal(sline .. "HitAreaRight"):Hide();
				getglobal(sline .. "HitArea"):Show();
				getglobal(sline .. "Note"):Hide();
				getglobal(sline .. "NoteButton"):Show();
				getglobal(sline .. "DeleteButton"):Show();
				getglobal(sline .. "DeleteText"):Show();
				getglobal(sline .. "Name"):SetWidth(200);
				local c = (player[2].Players[name] and player[2].Players[name].note) and 1 or 0.5;
				getglobal(sline .. "NoteButtonNormalTexture"):SetVertexColor(c, c, c);
				getglobal(sline .. "MouseOverLeft"):Hide();
				getglobal(sline .. "MouseOverRight"):Hide();
			elseif RaidTrackerFrame.type == "itemhistory" then
				getglobal(sline .. "HitAreaLeft"):Show();
				getglobal(sline .. "HitAreaRight"):Show();
				getglobal(sline .. "HitArea"):Hide();
				getglobal(sline .. "Note"):Show();
				getglobal(sline .. "NoteButton"):Hide();
				getglobal(sline .. "DeleteButton"):Hide();
				getglobal(sline .. "DeleteText"):Hide();
				getglobal(sline .. "Name"):SetWidth(130);
				getglobal(sline .. "Note"):SetText(player[3].player);
				getglobal(sline .. "MouseOver"):Hide();
			end
			line:Show();
		end
	end
	frame:Show();
end

function KA_RaidTracker_DetailScrollFrameEvents_Update()
	local frame = RT_DetailFrameEvents;	
	local self = RaidTracker;
	local db, options, L = self._db, self._options, self.L
	
	local events = frame.events;
	local maxlines = #events;
	FauxScrollFrame_Update(frame, maxlines, 11, 18);
	for i=1, 11 do
		local sline = "RT_DetailFrameEventsLine" .. i;
		local line = getglobal(sline);
		local id = i + FauxScrollFrame_GetOffset(frame);
		if id > maxlines then
			line:Hide();
		else
			local event = events[id];
			getglobal(sline .. "MouseOver"):Hide();
			getglobal(sline .. "HitArea"):Show();
			getglobal(sline .. "Boss"):SetText(event.boss);
			getglobal(sline .. "Time"):SetText(self:GetDisplayDate(event.time));
			line:Show();
		end
	end
	frame:Show();
end


function KA_RaidTracker_SelectRaid( raidid )
	local frame = RaidTrackerFrame

	KA_RaidTracker_GetPage();
	frame.selected = raidid;
	frame.type = "raids";

	KA_RaidTracker_UpdateView();
	KA_RaidTracker_Update();
end

function KA_RaidTracker_ShowInfo( player )
	local frame = RaidTrackerFrame
	
	KA_RaidTracker_GetPage();
	frame.type = "player";
	frame.player = player;
	frame.selected = nil;

	KA_RaidTracker_Update();
	KA_RaidTracker_UpdateView();
end

function KA_RaidTracker_SelectItem( sitem )
	local frame = RaidTrackerFrame

	KA_RaidTracker_GetPage();
	frame.type = "itemhistory";
	frame.itemname = sitem;
	frame.selected = nil;

	KA_RaidTracker_Update();
	KA_RaidTracker_UpdateView();
end

function KA_RaidTracker_GetPage()
	local frame = RaidTrackerFrame
	if frame.type or frame.itemname or frame.selected or frame.player then
		tinsert( frame.LastPage, {
				type = frame.type,
				itemname = frame.itemname,
				selected = frame.selected,
				player = frame.player,
			} );
	end
	if #frame.LastPage > 0 then
		RaidTrackerFrameBackButton:Enable();
	else
		RaidTrackerFrameBackButton:Disable();
	end
end

function KA_RaidTracker_GoBack()
	local frame = RaidTrackerFrame
	local t = tremove( frame.LastPage );
	if t then
		frame.type = t.type;
		frame.itemname = t.itemname;
		frame.selected = t.selected;
		frame.player = t.player;
		KA_RaidTracker_Update();
		KA_RaidTracker_UpdateView();
	end
	if #frame.LastPage > 0 then
		RaidTrackerFrameBackButton:Enable();
	else
		RaidTrackerFrameBackButton:Disable();
	end
end



